# -*- coding: utf-8 -*-
"""
/***************************************************************************
                          qpf.py  -  QGIS Plugin framework
                             -------------------
    begin                : 
    copyright            : (C) 2017 by alexander lisovenko
    email                : alexander.lisovenko@gmail.com
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/ 

"""

import os
import sys
import ConfigParser

from PyQt4.QtCore import *

from qgis.core import QgsMessageLog

version="0.0.4"

_fs_encoding = sys.getfilesystemencoding()
_this_script_path = unicode(os.path.abspath(__file__), _fs_encoding)
_this_script_dir = unicode(os.path.abspath(os.path.dirname(__file__)), _fs_encoding)

plugin_instance = None


class QGISPlugin(object):
    def __init__(self):
        config = ConfigParser.ConfigParser()
        
        self.__md_filename = os.path.join(
            _this_script_dir,
            "metadata.txt"
        )
        if not os.path.exists(self.__md_filename):
            raise ValueError("File metadata.txt not found")

        config.read([self.__md_filename])

        self.__name = config.get('general', 'name').strip()
        self.__version = config.get('general', 'version')
        
        self.__pack_name = self.__generate_plugin_pack_name()
        
        global plugin_instance
        plugin_instance = self

    def __generate_plugin_pack_name(self):
        pl_dir_name = ""
        
        i = 0
        if ( self.__name.find(' ') > 0 ):
            while ( i < len(self.__name) ):
                if ( self.__name[i] == ' ' ):
                    while (self.__name[i] == ' ' ):
                        i += 1
                    pl_dir_name += '_'

                pl_dir_name += self.__name[i].lower()
                i += 1
        else:
            while ( i < len(self.__name) ):
                if ( self.__name[i].isupper() ):
                    pl_dir_name += '_'
                    while( self.__name[i].isupper() ):
                        pl_dir_name += self.__name[i].lower()
                        i += 1

                pl_dir_name += self.__name[i]
                i += 1

        return pl_dir_name.strip('_')
    

    @property
    def pack_name(self):
        return self.__pack_name
    

    @property
    def name(self):
        return self.__name


    @property
    def version(self):
        return self.__version


    @property
    def i18nPath(self):
        return os.path.join(_this_script_dir, "i18n")
    
    @staticmethod
    def log(msg, level=QgsMessageLog.INFO):
        global plugin_instance
        QgsMessageLog.logMessage(
            msg,
            plugin_instance.name,
            level
        )

    @staticmethod
    def rootDir():
        return _this_script_dir

    @staticmethod
    def currentLocal():
        return QSettings().value('locale/userLocale')[0:2]

if __name__ == "__main__":
    import re
    import json
    import pprint
    import shutil
    import urllib
    import urllib2
    import zipfile
    import argparse
    import StringIO
    import subprocess

    parser = argparse.ArgumentParser(description='Make qgis python plugin.')
    
    parser.add_argument('-v', '--version', action="store_true", help='QPF Version')
    
    self_update_cmd_name = 'self_update'
    install_cmd_name = 'install'
    pack_cmd_name = 'pack'
    make_cmd_name = 'make'
    ts_cmd_name = 'ts'
    qm_cmd_name = 'qm'
    
    subparsers = parser.add_subparsers(help='commands help', dest='command')
    
    self_update = subparsers.add_parser(self_update_cmd_name, help='Update qpf')
    
    parser_install = subparsers.add_parser(install_cmd_name, help='Install plugin to dir')
    parser_install.add_argument('install_dir', type=unicode, help='Destanation dir')
    
    parser_pack = subparsers.add_parser(pack_cmd_name, help='Make zip with plugin')
    parser_pack.add_argument('--exclude', type=unicode, help='Exclude dir name')
    
    parser_make = subparsers.add_parser(make_cmd_name, help='Make all uis, rcs, i18 and other')
    pack_type_release = "Release" 
    pack_type_develop = "Develop" 
    parser_make.add_argument('type', choices=[pack_type_release, pack_type_develop], type=unicode, help='Release or Develop')

    parser_ts = subparsers.add_parser(ts_cmd_name, help='Make ts file from all files in plugin')
    parser_ts.add_argument('--black_list', type=unicode, help='list of exclude dirs and files')

    parser_qm = subparsers.add_parser(qm_cmd_name, help='Make qm files from qm file in ./i18n dir')
    

    args = parser.parse_args()
    
    def get_git_version():
        cmd = ['git', 'describe', '--tags', '--long', '--always']

        cwd = os.getcwd()
        os.chdir(_this_script_dir)
        res = subprocess.check_output(cmd, shell=True)
        os.chdir(cwd)
        m = re.match(r"(\d+)\.(\d+)\.(\d+)-(\d+).*", res)
        while m is None:
            version_str = raw_input("Input version string as x.y.z-b: ")
            m = re.match(r"(\d+)\.(\d+)\.(\d+)-(\d+).*", version_str)

        major, minor, patch, commit = m.groups()
        print major, minor, patch, commit
        return major, minor, patch, commit

    def create_metadata(build_type=pack_type_develop):
        md_file_in = os.path.join(_this_script_dir, "metadata.txt.in")
        if not os.path.exists(md_file_in):
            print "Cann't create metadata.txt. There in no metadata.txt.in neare this script!"
            return

        major, minor, patch, commit = get_git_version()

        version = "%s.%s.%s-%s" % (major, minor, patch, commit)
        if build_type == pack_type_release:
            version = "%s.%s.%s" % (major, minor, patch)

        changelog = "%s\n" % version
        changelog_file = os.path.join(_this_script_dir, "changelog")
        with open(changelog_file) as f:
            # skip first line
            first_line = f.readline()

            for line in f:
                changelog += "\t"+line

        mdfilename = os.path.join(_this_script_dir, u"metadata.txt")
        with open(mdfilename, "w") as f:
            with open(md_file_in) as f_src:
                for line in f_src:
                    line = line.format(
                            VERSION=version,
                            CHANGELOG=changelog,
                    )
                    f.write(line)


    def install(install_dir):
        plugin_dir_name = QGISPlugin().pack_name

        print "install ..."

        if not os.path.exists(install_dir):
            print "Directory %s not found" % install_dir
            return

        install_dir = os.path.join(install_dir, plugin_dir_name)

        ignore_list = (
            '.gitignore',
            '.pyc',
            '.ts',
            '.qrc',
            '.in',
            'changelog',
        )

        def copytree(src, dst):
            if not os.path.exists(dst):
                os.makedirs(dst)
            for item in os.listdir(src):
                if item.endswith(ignore_list):
                    continue
                s = os.path.join(src, item)
                d = os.path.join(dst, item)
                if os.path.isdir(s):
                    copytree(s, d)
                else:
                    if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                        print "install %s" % d
                        shutil.copy2(s, d)

        copytree(_this_script_dir, install_dir)

    def pack(exclude_dir):
        print "packing ..."
        pl = QGISPlugin()

        plugin_dir_name = pl.pack_name
        plugin_version = pl.version
        zipFile = zipfile.ZipFile(
            os.path.join(
                os.getcwd(),
                plugin_dir_name + "." + plugin_version + ".zip"
            ),
            'w'
        )
        ignore_list = (
            '.gitignore',
            '.pyc',
            '.ts',
            '.qrc',
            '.in',
            'changelog',
        )
        for root, dirs, files in os.walk(_this_script_dir):
            if root.find('.git') != -1:
                continue

            if exclude_dir is not None:
                if root.find(exclude_dir) != -1:
                    continue

            for f in files:
                if f.endswith(ignore_list):
                    continue
                filename = os.path.join(root, f)
                zipFile.write(
                    filename,
                    os.path.join(
                        plugin_dir_name,
                        os.path.relpath(filename, _this_script_dir)
                    )
                )

    def generate_ts(black_list):
        print "generate ts ..."
        pl = QGISPlugin()
        if not os.path.exists(pl.i18nPath):
            os.makedirs(pl.i18nPath)

        ts_file = os.path.join(
            pl.i18nPath,
            "%s_ru.ts" % pl.pack_name
        )

        white_list = (
            '.py',
            '.ui',
        )

        target_files = []
        for root, dirs, files in os.walk(_this_script_dir):
            if root.find('.git') != -1:
                continue

            if black_list is not None and root.find(black_list) != -1:
                continue

            for f in files:
                if not f.endswith(white_list):
                    continue
                filename = os.path.join(root, f)
                target_files.append(filename)

        cmd = ['pylupdate4']
        cmd.extend(target_files)
        cmd.append("-ts")
        cmd.append(ts_file)

        pprint.pprint(cmd)
        subprocess.call(cmd, shell=True)


    def make_qm():
        print "making ts ..."
        pl = QGISPlugin()
        i18n_files = [f for f in os.listdir(pl.i18nPath) if os.path.splitext(f)[1] == ".ts"]

        for i18n_file in i18n_files:
            ts_file = os.path.join(pl.i18nPath, i18n_file)
            print "Process %s ..." % ts_file
            subprocess.call([
                'lrelease',
                ts_file
            ])

    def make():
        create_metadata(args.type)

    if args.version:
        print version

    if args.command == self_update_cmd_name:
        # url = "https://gitlab.com/alisovenko/qpf/raw/master/qpf.py"
        url = 'file:' + urllib.pathname2url(r'E:\\dev\\qgis.plugin_framework\\qpf.py')
        with urllib2.urlopen(url).fp as new_qpf:
            if os.path.normcase(new_qpf.name) != os.path.normcase(_this_script_path):
                with open(_this_script_path, "w") as f_dst:
                    for line in new_qpf:
                        f_dst.write(line.rstrip('\n'))
            else:
                print "Allready up to date!"

    if args.command == install_cmd_name:
        install(args.install_dir)
    
    if args.command == pack_cmd_name:
        pack(args.exclude)

    if args.command == ts_cmd_name:
        generate_ts(args.black_list)

    if args.command == qm_cmd_name:
        make_qm()

    if args.command == make_cmd_name:
        make()